@@ -4,16 +4,13 @@ import time
 
 import django.db.transaction
 from django.core.management.base import BaseCommand
-from stellar_sdk.exceptions import BaseRequestError
+from stellar_sdk.exceptions import BaseRequestError, NotFoundError, ConnectionError
 
 from polaris import settings
 from polaris.models import Transaction
 from polaris.utils import getLogger, maybe_make_callback
 from polaris.integrations import registered_deposit_integration as rdi
-from polaris.management.commands.poll_pending_deposits import (
-    PendingDeposits,
-    MultiSigTransactions,
-)
+from polaris.management.commands.poll_pending_deposits import PendingDeposits
 
 
 logger = getLogger(__name__)
@@ -33,20 +30,20 @@ class Command(BaseCommand):
         signal.signal(signal.SIGTERM, self.exit_gracefully)
 
     @staticmethod
-    def exit_gracefully(sig, frame):
+    def exit_gracefully(sig, frame):  # pragma: no cover
         logger.info("Exiting check_trustlines...")
         module = sys.modules[__name__]
         module.TERMINATE = True
 
     @staticmethod
-    def sleep(seconds):
+    def sleep(seconds):  # pragma: no cover
         module = sys.modules[__name__]
         for _ in range(seconds):
             if module.TERMINATE:
                 break
             time.sleep(1)
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser):  # pragma: no cover
         parser.add_argument(
             "--loop",
             action="store_true",
@@ -60,7 +57,7 @@ class Command(BaseCommand):
             "Defaults to {}.".format(DEFAULT_INTERVAL),
         )
 
-    def handle(self, *_args, **options):
+    def handle(self, *_args, **options):  # pragma: no cover
         module = sys.modules[__name__]
         if options.get("loop"):
             while True:
@@ -128,19 +125,33 @@ class Command(BaseCommand):
                         f"Account {account['id']} has established a trustline for "
                         f"{balance['asset_code']}:{balance['asset_issuer']}"
                     )
-                    if MultiSigTransactions.requires_multisig(transaction):
-                        MultiSigTransactions.save_as_pending_signatures(transaction)
+                    try:
+                        requires_multisig = PendingDeposits.requires_multisig(
+                            transaction
+                        )
+                    except NotFoundError:
+                        PendingDeposits.handle_error(
+                            transaction,
+                            f"{transaction.asset.code} distribution account "
+                            f"{transaction.asset.distribution_account} does not exist",
+                        )
+                        break
+                    except ConnectionError:
+                        logger.error("Failed to connect to Horizon")
+                        transaction.pending_execution_attempt = False
+                        transaction.save()
+                        break
+                    if requires_multisig:
+                        PendingDeposits.save_as_pending_signatures(transaction)
                         break
 
                     try:
                         success = PendingDeposits.submit(transaction)
                     except Exception as e:
                         logger.exception("submit() threw an unexpected exception")
-                        transaction.status_message = str(e)
-                        transaction.status = Transaction.STATUS.error
-                        transaction.pending_execution_attempt = False
-                        transaction.save()
-                        maybe_make_callback(transaction)
+                        PendingDeposits.handle_error(
+                            transaction, f"{e.__class__.__name__}: {str(e)}"
+                        )
                         break
 
                     if success:
