@@ -7,7 +7,6 @@ from decimal import Decimal, DecimalException
 from urllib.parse import urlencode
 
 from django.urls import reverse
-from django.core.exceptions import ValidationError
 from django.shortcuts import redirect
 from django.views.decorators.clickjacking import xframe_options_exempt
 from django.utils.translation import gettext as _
@@ -125,6 +124,20 @@ def post_interactive_deposit(request: Request) -> Response:
     if form.is_valid():
         if issubclass(form.__class__, TransactionForm):
             transaction.amount_in = form.cleaned_data["amount"]
+            transaction.amount_fee = registered_fee_func(
+                {
+                    "amount": transaction.amount_in,
+                    "type": form.cleaned_data.get("type"),
+                    "operation": settings.OPERATION_DEPOSIT,
+                    "asset_code": asset.code,
+                }
+            )
+            if settings.ADDITIVE_FEES_ENABLED:
+                transaction.amount_in += transaction.amount_fee
+            transaction.amount_out = round(
+                transaction.amount_in - transaction.amount_fee,
+                asset.significant_decimals,
+            )
             transaction.save()
 
         rdi.after_form_validation(form, transaction)
@@ -141,12 +154,14 @@ def post_interactive_deposit(request: Request) -> Response:
             return redirect(f"{url}?{urlencode(args)}")
 
         else:  # Last form has been submitted
-            logger.info(
-                f"Finished data collection and processing for transaction {transaction.id}"
-            )
+            logger.info(f"Finished data collection for transaction {transaction.id}")
             invalidate_session(request)
-            transaction.status = Transaction.STATUS.pending_user_transfer_start
-            transaction.save()
+            transaction.refresh_from_db()
+            if transaction.status != transaction.STATUS.pending_anchor:
+                transaction.status = Transaction.STATUS.pending_user_transfer_start
+                transaction.save()
+            else:
+                logger.info(f"Transaction {transaction.id} is pending KYC approval")
             args = {"id": transaction.id, "initialLoad": "true"}
             if callback:
                 args["callback"] = callback
@@ -184,6 +199,7 @@ def post_interactive_deposit(request: Request) -> Response:
             operation=settings.OPERATION_DEPOSIT,
             asset=asset,
             use_fee_endpoint=registered_fee_func != calculate_fee,
+            additive_fees_enabled=settings.ADDITIVE_FEES_ENABLED,
         )
         return Response(content, template_name="polaris/deposit.html", status=400)
 
@@ -301,6 +317,7 @@ def get_interactive_deposit(request: Request) -> Response:
         asset=asset,
         use_fee_endpoint=registered_fee_func != calculate_fee,
         org_logo_url=toml_data.get("DOCUMENTATION", {}).get("ORG_LOGO"),
+        additive_fees_enabled=settings.ADDITIVE_FEES_ENABLED,
     )
 
     return Response(content, template_name="polaris/deposit.html")
