@@ -120,6 +120,20 @@ def post_interactive_withdraw(request: Request) -> Response:
     elif form.is_valid():
         if issubclass(form.__class__, TransactionForm):
             transaction.amount_in = form.cleaned_data["amount"]
+            transaction.amount_fee = registered_fee_func(
+                {
+                    "amount": transaction.amount_in,
+                    "type": form.cleaned_data.get("type"),
+                    "operation": settings.OPERATION_WITHDRAWAL,
+                    "asset_code": asset.code,
+                }
+            )
+            if settings.ADDITIVE_FEES_ENABLED:
+                transaction.amount_in += transaction.amount_fee
+            transaction.amount_out = round(
+                transaction.amount_in - transaction.amount_fee,
+                asset.significant_decimals,
+            )
             transaction.save()
 
         rwi.after_form_validation(form, transaction)
@@ -141,6 +155,9 @@ def post_interactive_withdraw(request: Request) -> Response:
             )
             invalidate_session(request)
 
+            # ensure all changes from `after_form_validation()` have been saved to the db
+            transaction.refresh_from_db()
+
             # Add memo now that interactive flow is complete
             #
             # We use the transaction ID as a memo on the Stellar transaction for the
@@ -152,7 +169,11 @@ def post_interactive_withdraw(request: Request) -> Response:
             transaction_id_hex = transaction.id.hex
             padded_hex_memo = "0" * (64 - len(transaction_id_hex)) + transaction_id_hex
             transaction.memo = memo_hex_to_base64(padded_hex_memo)
-            transaction.status = Transaction.STATUS.pending_user_transfer_start
+
+            if transaction.status != transaction.STATUS.pending_anchor:
+                transaction.status = Transaction.STATUS.pending_user_transfer_start
+            else:
+                logger.info(f"Transaction {transaction.id} is pending KYC approval")
             transaction.save()
             args = {"id": transaction.id, "initialLoad": "true"}
             if callback:
@@ -193,6 +214,7 @@ def post_interactive_withdraw(request: Request) -> Response:
             asset=asset,
             use_fee_endpoint=registered_fee_func != calculate_fee,
             org_logo_url=toml_data.get("DOCUMENTATION", {}).get("ORG_LOGO"),
+            additive_fees_enabled=settings.ADDITIVE_FEES_ENABLED,
         )
         return Response(content, template_name="polaris/withdraw.html", status=400)
 
@@ -308,6 +330,7 @@ def get_interactive_withdraw(request: Request) -> Response:
         operation=settings.OPERATION_WITHDRAWAL,
         asset=asset,
         use_fee_endpoint=registered_fee_func != calculate_fee,
+        additive_fees_enabled=settings.ADDITIVE_FEES_ENABLED,
     )
 
     return Response(content, template_name="polaris/withdraw.html")
